/* curl_helpers.c -- Some routins indent to simplify the use of curl
 * 
 * Copyright (C) 2004 H Zero Seven
 *
 * Authors: Enrico Kern <phantom@h07.org>
 *          Christian Volmering <risk@h07.org>
 * 
 * This file is part of the aexpl-ng distribution.
 * 
 * The aexpl-ng distribution is free software; you can redistribute it 
 * and/or modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 of 
 * the License, or (at your option) any later version.
 *
 * The aexpl-ng distribution is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied 
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the aexpl-ng distribution; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 * MA 02111-1307 USA
 */

#include "config.h"
#include "proxy.h"

#include <stdio.h>
#include <curl/curl.h>
#include <curl/types.h>
#include <curl/easy.h>

#ifdef WITH_DMALLOC
#  include <dmalloc.h>
#endif

struct _web_file_t
{
  char *dest_file;
  char *buffer;
  size_t buffer_size;
  FILE *stream;
};

extern proxy_config_t proxy_config;
extern char ca_cert[PATH_MAX];

/* {{{ _write_file */
static int
_write_file (void *buffer, size_t size, size_t nmemb, void *user_ptr)
{
  struct _web_file_t *web_file = (struct _web_file_t *) user_ptr;

  if (!web_file)
    return (-1);

  if (web_file->buffer)
    return (snprintf
	    (web_file->buffer, web_file->buffer_size, "%s", (char *) buffer));

  if (web_file->dest_file)
    {
      if (!web_file->stream)
	{
	  if (!(web_file->stream = fopen (web_file->dest_file, "wb")))
	    {
	      perror ("Unable to open destination file");

	      return (-1);
	    }
	}

      return (fwrite (buffer, size, nmemb, web_file->stream));
    }

  return (-1);
}

/* }}} */

/* {{{ curl_get */
int
curl_get (const char *url, char *dest_file, char *dest_buffer,
	  size_t buffer_size)
{
  CURL *curl;
  CURLcode res;
  char user_agent[100];
  struct _web_file_t webfile = { dest_file, dest_buffer, buffer_size, NULL };
  char err_buff[CURL_ERROR_SIZE];

  curl_global_init (CURL_GLOBAL_DEFAULT);

  curl = curl_easy_init ();

  if (curl)
    {
      snprintf (user_agent, 100, "AntiExploit (%s)", VERSION);

      curl_easy_setopt (curl, CURLOPT_ERRORBUFFER, err_buff);
      curl_easy_setopt (curl, CURLOPT_URL, url);
      curl_easy_setopt (curl, CURLOPT_USERAGENT, user_agent);
      curl_easy_setopt (curl, CURLOPT_WRITEFUNCTION, _write_file);

#ifdef CURL_SSL
      curl_easy_setopt (curl, CURLOPT_SSL_VERIFYHOST, 1);
      curl_easy_setopt (curl, CURLOPT_SSL_VERIFYPEER, 1);
      curl_easy_setopt (curl, CURLOPT_CAINFO, ca_cert);
#endif

      // Configure proxy settings
      if (P_ENABLED (proxy_config))
	{
	  int type;

	  if ((type = P_TYPE (proxy_config)) == -1)
	    {
	      fprintf (stderr, "Error: %s\n", "Invalid proxy type.");

	      return (0);
	    }

	  curl_easy_setopt (curl, CURLOPT_PROXY, P_HOST (proxy_config));
	  curl_easy_setopt (curl, CURLOPT_PROXYUSERPWD,
			    P_LOGIN (proxy_config));
	  curl_easy_setopt (curl, CURLOPT_PROXYTYPE, type);
#if (LIBCURL_VERSION_NUM >= 0x070a07)
	  curl_easy_setopt (curl, CURLOPT_PROXYAUTH, CURLAUTH_ANY);
#endif
	}

      curl_easy_setopt (curl, CURLOPT_WRITEDATA, &webfile);
      res = curl_easy_perform (curl);

      curl_easy_cleanup (curl);

      if (res != CURLE_OK)
	{
	  fprintf (stderr, "Error: %s\n", err_buff);

	  return (0);
	}
    }

  if (webfile.stream)
    fclose (webfile.stream);

  curl_global_cleanup ();

  return (1);
}

/* }}} */

/*
 * Local variables:
 * tab-width: 8
 * c-basic-offset: 8
 * End:
 * vim600: noet nonu sw=8 ts=8 fdm=marker
 * vim<600: noet nonu sw=8 ts=8
 */
